#!/usr/bin/env python

__description__ = 'MSG summary plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.1'
__date__ = '2020/09/11'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2020/08/18: start
  2020/09/11: added body

Todo:
"""

import struct
import re
import optparse

def StartsWithGetRemainder(strIn, strStart):
    if strIn.startswith(strStart):
        return True, strIn[len(strStart):]
    else:
        return False, None

class cAttachment(object):
    def __init__(self, name):
        self.name = name

class cMSG(cPluginParentOle):
    indexQuiet = True
    name = 'MSG summary plugin'

    def PreProcess(self):
        self.dAttachments = {}
        self.streamIndexCounter = 0

    def Process(self, name, stream):
        self.streamIndexCounter += 1
        found, number = StartsWithGetRemainder(name[0], '__attach_version1.0_#')
        if found:
            nameAttachment = name[0]
            numberAttachment = int(number)
            if not numberAttachment in self.dAttachments:
                self.dAttachments[numberAttachment] = cAttachment(nameAttachment)
                self.dAttachments[numberAttachment].number = numberAttachment
            found, code = StartsWithGetRemainder(name[1], '__substg1.0_')
            if found:
                if code == '37010102':
                    self.dAttachments[numberAttachment].data = stream
                    self.dAttachments[numberAttachment].streamIndex = self.streamIndexCounter
                elif code.startswith('3707'):
                    self.dAttachments[numberAttachment].longfilename = stream.decode('utf16') if code.endswith('001F') else stream.decode()
                elif code.startswith('370E'):
                    self.dAttachments[numberAttachment].mimetag = stream.decode('utf16') if code.endswith('001F') else stream.decode()

        found, code = StartsWithGetRemainder(name[0], '__substg1.0_')
        if found:
            if code.startswith('0037'):
                self.subject = stream.decode('utf16') if code.endswith('001F') else stream.decode()
            elif code.startswith('007D'):
                self.header = stream.decode('utf16') if code.endswith('001F') else stream.decode()
                self.headerStreamIndex = self.streamIndexCounter
                for line in self.header.split('\n'):
                    line = line.rstrip('\r')
                    found, value = StartsWithGetRemainder(line, 'Date: ')
                    if found:
                        self.date = value
                    found, value = StartsWithGetRemainder(line, 'To: ')
                    if found:
                        self.to = value
                    found, value = StartsWithGetRemainder(line, 'From: ')
                    if found:
                        self.from_ = value
            elif code.startswith('1000'):
                self.body = stream.decode('utf16') if code.endswith('001F') else stream.decode()
                self.bodyStreamIndex = self.streamIndexCounter

    def PostProcess(self):
        oParser = optparse.OptionParser()
        oParser.add_option('-j', '--json', action='store_true', default=False, help='Produce JSON output')
        oParser.add_option('-b', '--body', action='store_true', default=False, help='Print body')
        oParser.add_option('-H', '--header', action='store_true', default=False, help='Print header')
        (options, args) = oParser.parse_args(self.options.split(' '))

        sha256 = hashlib.sha256(self.data).hexdigest()
        if options.json:
            jsondata = {'sha256': sha256, 'subject': self.subject, 'date': self.date, 'to': self.to, 'from': self.from_, 'attachments': [{'index': key, 'longfilename': value.longfilename, 'mimetag': value.mimetag, 'size': len(value.data), 'sha256': hashlib.sha256(value.data).hexdigest(), 'magichex': binascii.b2a_hex(value.data[:4]).decode()} for key, value in self.dAttachments.items()]}
            print(json.dumps(jsondata))
        else:
            print('Sample email: sha256 %s' % sha256)
            print('Header stream index: %d' % self.headerStreamIndex)
            print('Subject: %s' % self.subject)
            print('Date: %s' % self.date)
            print('To: %s' % self.to)
            print('From: %s' % self.from_)
            print('Body stream index: %d' % self.bodyStreamIndex)
            for index in self.dAttachments.keys():
                print('Attachment %d (stream index %d): %s %s %d %s' % (index, self.dAttachments[index].streamIndex, self.dAttachments[index].longfilename, self.dAttachments[index].mimetag, len(self.dAttachments[index].data), hashlib.sha256(self.dAttachments[index].data).hexdigest()))
            if options.body:
                print('Body:')
                print(self.body)
            if options.header:
                print('Header:')
                print(self.header)

AddPlugin(cMSG)



#!/usr/bin/env python

__description__ = 'VBA declare/createobject plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.3'
__date__ = '2019/11/25'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2015/10/25: start
  2015/11/04: added keywords
  2019/11/25: 0.0.3 update for GetObject, Callbyname and Shell

Todo:
"""

import re

def ContainsString(stringsToFind, containingString):
    containingString = containingString.lower()
    for stringToFind in stringsToFind:
        if stringToFind.lower() in containingString:
            return True
    return False
            
def ExtractDeclareFunctionSub(line):
    if ContainsString(['declare'], line):
        oMatch = re.search(r'(function|sub)\s+(\S+)\s+', line, re.I)
        if oMatch == None:
            return None
        return oMatch.group(2)
    else:
        return None

def ExtractSetObjectVariable(line):
    if ContainsString(['createobject', 'getobject'], line):
        oMatch = re.search(r'(\S+)\s*=\s*(CreateObject|GetObject)', line, re.I)
        if oMatch == None:
            return None
        return oMatch.group(1)
    else:
        return None

class cVBADCO(cPluginParent):
    macroOnly = True
    name = 'VBA DCO (Declare/CreateObject) plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options
        self.ran = False

    def Analyze(self):
        self.ran = True

        oREDCO = re.compile(r'\b(declare|createobject|getobject|callbyname|shell)\b', re.I)
        result = [line.strip() for line in self.stream.split('\n') if re.search(oREDCO, line) != None]
        keywords = []
        for line in result:
            keyword = ExtractDeclareFunctionSub(line)
            if keyword != None and not keyword in keywords:
                keywords.append(keyword)
            keyword = ExtractSetObjectVariable(line)
            if keyword != None and not keyword in keywords:
                keywords.append(keyword)
        keywordLines = [line.strip() for line in self.stream.split('\n') if ContainsString(keywords, line)]
        if keywordLines != []:
            result.append('-' * 80)
            result.extend(keywordLines)
        return result

AddPlugin(cVBADCO)



#!/usr/bin/env python

__description__ = 'MSG plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.4'
__date__ = '2019/11/04'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2017/12/16: start
  2017/12/17: 0.0.2 added options -d and -a
  2017/12/30: removed option -a (becomes default) and -d; and added option -n
  2018/07/14: 0.0.3 added option -k
  2019/11/04: 0.0.4 added tag 1009

Todo:
"""

import struct
import re
import optparse

class cMSG(cPluginParent):
    macroOnly = False
    indexQuiet = True
    name = 'MSG plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options
        self.ran = False

    def Analyze(self):
        result = []

        #http://www.fileformat.info/format/outlookmsg/
        dCodes = {
          '001A': 'Message class',
          '0037': 'Subject',
          '003D': 'Subject prefix',
          '0040': 'Received by name',
          '0042': 'Sent repr name',
          '0044': 'Rcvd repr name',
          '004D': 'Org author name',
          '0050': 'Reply rcipnt names',
          '005A': 'Org sender name',
          '0064': 'Sent repr adrtype',
          '0065': 'Sent repr email',
          '0070': 'Topic',
          '0075': 'Rcvd by adrtype',
          '0076': 'Rcvd by email',
          '0077': 'Repr adrtype',
          '0078': 'Repr email',
          '007D': 'Message header',
          '0C1A': 'Sender name',
          '0C1E': 'Sender adr type',
          '0C1F': 'Sender email',
          '0E02': 'Display BCC',
          '0E03': 'Display CC',
          '0E04': 'Display To',
          '0E1D': 'Subject (normalized)',
          '0E28': 'Recvd account1(?)',
          '0E29': 'Recvd account2(?)',
          '1000': 'Message body',
          '1008': 'RTF sync body tag',
          '1009': 'RTF Compressed',
          '1035': 'Message ID (?)',
          '1046': 'Sender email(?)',
          '3001': 'Display name',
          '3002': 'Address type',
          '3003': 'Email address',
          '39FE': '7-bit email (?)',
          '39FF': '7-bit display name',
          '3701': 'Attachment data',
          '3703': 'Attach extension',
          '3704': 'Attach filename',
          '3707': 'Attach long filename',
          '370E': 'Attach mime tag',
          '3712': 'Attach ID (?)',
          '3A00': 'Account',
          '3A02': 'Callback phone no',
          '3A05': 'Generation',
          '3A06': 'Given name',
          '3A08': 'Business phone',
          '3A09': 'Home phone',
          '3A0A': 'Initials',
          '3A0B': 'Keyword',
          '3A0C': 'Language',
          '3A0D': 'Location',
          '3A11': 'Surname',
          '3A15': 'Postal address',
          '3A16': 'Company name',
          '3A17': 'Title',
          '3A18': 'Department',
          '3A19': 'Office location',
          '3A1A': 'Primary phone',
          '3A1B': 'Business phone 2',
          '3A1C': 'Mobile phone',
          '3A1D': 'Radio phone no',
          '3A1E': 'Car phone no',
          '3A1F': 'Other phone',
          '3A20': 'Transmit dispname',
          '3A21': 'Pager',
          '3A22': 'User certificate',
          '3A23': 'Primary Fax',
          '3A24': 'Business Fax',
          '3A25': 'Home Fax',
          '3A26': 'Country',
          '3A27': 'Locality',
          '3A28': 'State/Province',
          '3A29': 'Street address',
          '3A2A': 'Postal Code',
          '3A2B': 'Post Office Box',
          '3A2C': 'Telex',
          '3A2D': 'ISDN',
          '3A2E': 'Assistant phone',
          '3A2F': 'Home phone 2',
          '3A30': 'Assistant',
          '3A44': 'Middle name',
          '3A45': 'Dispname prefix',
          '3A46': 'Profession',
          '3A48': 'Spouse name',
          '3A4B': 'TTYTTD radio phone',
          '3A4C': 'FTP site',
          '3A4E': 'Manager name',
          '3A4F': 'Nickname',
          '3A51': 'Business homepage',
          '3A57': 'Company main phone',
          '3A58': 'Childrens names',
          '3A59': 'Home City',
          '3A5A': 'Home Country',
          '3A5B': 'Home Postal Code',
          '3A5C': 'Home State/Provnce',
          '3A5D': 'Home Street',
          '3A5F': 'Other adr City',
          '3A60': 'Other adr Country',
          '3A61': 'Other adr PostCode',
          '3A62': 'Other adr Province',
          '3A63': 'Other adr Street',
          '3A64': 'Other adr PO box',
          '3FF7': 'Server   (?)',
          '3FF8': 'Creator1 (?)',
          '3FFA': 'Creator2 (?)',
          '3FFC': 'To email (?)',
          '403D': 'To adrtype(?)',
          '403E': 'To email (?)',
          '5FF6': 'To       (?)'
        }

        dTypes = {'001E': 'ASC', '001F': 'UNI', '0102': 'BIN'}

        oParser = optparse.OptionParser()
        oParser.add_option('-n', '--nodecode', action='store_true', default=False, help='Do not decode data')
        oParser.add_option('-k', '--known', action='store_true', default=False, help='Only display known hexcodes')
        (options, args) = oParser.parse_args(self.options.split(' '))

        self.ran = True
        oMatch = re.search('_[0-9A-F]{8}', self.streamname[-1])
        if oMatch != None:
            hexcode = oMatch.group()[1:5]
            hextype = oMatch.group()[5:]
            if hextype == '001F':
                decoded = self.stream.decode('utf16')
            elif hextype == '0102':
                decoded = repr(self.stream)
            else:
                decoded = ''
            line = '%s %s: %s %s' % (hexcode, hextype, dTypes.get(hextype, '?  '), dCodes.get(hexcode, '?'))
            if options.nodecode:
                pass
            else:
                line = (line + ' ' * 40)[0:40] + ' ' + decoded[0:40]
            if not options.known or options.known and hexcode in dCodes:
                result.append(line)
        
        return result

AddPlugin(cMSG)
